@@ -3,7 +3,7 @@
 # abstraction for non-interactive backends (like chroot, qemu)
 #
 
-from DistUpgradeConfigParser import DistUpgradeConfig
+from DistUpgrade.DistUpgradeConfigParser import DistUpgradeConfig
 
 import ConfigParser
 import os
@@ -35,12 +35,13 @@ class UpgradeTestBackend(object):
 
     apt_options = ["-y","--allow-unauthenticated"]
 
-    def __init__(self, profile, basefiledir):
+    def __init__(self, profiledir, resultdir=""):
         " init the backend with the given profile "
         # init the dirs
-        assert(profile != None)
-        self.resultdir = os.path.abspath(os.path.join(os.path.dirname(profile),"result"))
-        self.basefilesdir = os.path.abspath(basefiledir)
+        assert(profiledir != None)
+        profiledir = os.path.normpath(profiledir)
+        profile = os.path.join(os.path.abspath(profiledir), "DistUpgrade.cfg")
+        self.upgradefilesdir = "./DistUpgrade"
         # init the rest
         if os.path.exists(profile):
             self.profile = os.path.abspath(profile)
@@ -48,9 +49,18 @@ class UpgradeTestBackend(object):
                                             name=os.path.basename(profile))
         else:
             raise IOError, "Can't find profile '%s' (%s) " % (profile, os.getcwd())
+        base_resultdir = self.config.getWithDefault("NonInteractive",
+                                                    "ResultDir",
+                                                    "results-upgrade-tester")
+        self.resultdir = os.path.abspath(
+            os.path.join(base_resultdir, profiledir.split("/")[-1]))
+        if not os.path.exists(self.resultdir):
+            os.makedirs(self.resultdir)
         
         self.fromDist = self.config.get("Sources","From")
-        if self.config.has_option("NonInteractive","Proxy"):
+        if "http_proxy" in os.environ and not self.config.has_option("NonInteractive","Proxy"):
+	    self.config.set("NonInteractive","Proxy", os.environ["http_proxy"])
+        elif self.config.has_option("NonInteractive","Proxy"):
             proxy=self.config.get("NonInteractive","Proxy")
             os.putenv("http_proxy",proxy)
         os.putenv("DEBIAN_FRONTEND","noninteractive")
@@ -63,7 +73,6 @@ class UpgradeTestBackend(object):
         # run from cron)
         os.environ["PATH"] = "/usr/sbin:/usr/bin:/sbin:/bin"
 
-
     def installPackages(self, pkgs):
         """
         install packages in the image
