@@ -27,15 +27,12 @@ import pygtk
 pygtk.require('2.0')
 import gtk
 import gtk.gdk
-import gtk.glade
-try:
-    import gconf
-except:
-    import fakegconf as gconf
+import gconf
 import gobject
+import glib
 
 import warnings
-warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
+warnings.filterwarnings("ignore", "Accessed deprecated property", DeprecationWarning)
 import apt
 import apt_pkg
 
@@ -64,6 +61,7 @@ import dbus.service
 import dbus.glib
 
 import GtkProgress
+import backend
 
 from gettext import gettext as _
 from gettext import ngettext
@@ -76,7 +74,7 @@ from Core.MetaRelease import Dist
 
 from DistUpgradeFetcher import DistUpgradeFetcherGtk
 from ChangelogViewer import ChangelogViewer
-from SimpleGladeApp import SimpleGladeApp
+from SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from HelpViewer import HelpViewer
 from MetaReleaseGObject import MetaRelease
 
@@ -104,15 +102,21 @@ class UpdateManagerDbusControler(dbus.service.Object):
         self.parent.window_main.present()
         return True
 
-class UpdateManager(SimpleGladeApp):
+class UpdateManager(SimpleGtkbuilderApp):
 
   def __init__(self, datadir, options):
     self.setupDbus()
     gtk.window_set_default_icon_name("update-manager")
 
     self.datadir = datadir
-    SimpleGladeApp.__init__(self, datadir+"glade/UpdateManager.glade",
-                            None, domain="update-manager")
+    SimpleGtkbuilderApp.__init__(self, datadir+"glade/UpdateManager.ui",
+                                 "update-manager")
+    gettext.bindtextdomain("update-manager", "/usr/share/locale")
+    gettext.textdomain("update-manager")
+    try:
+        locale.setlocale(locale.LC_ALL, "")
+    except:
+        logging.exception("setlocale failed")
 
     self.image_logo.set_from_icon_name("update-manager", gtk.ICON_SIZE_DIALOG)
     self.window_main.set_sensitive(False)
@@ -193,6 +197,9 @@ class UpdateManager(SimpleGladeApp):
                                               self.progressbar_cache,
                                               self.label_cache,
                                               self.window_main)
+
+    #set minimum size to prevent the headline label blocking the resize process
+    self.window_main.set_size_request(500,-1) 
     # restore state
     self.restore_state()
     # deal with no-focus-on-map
@@ -201,14 +208,36 @@ class UpdateManager(SimpleGladeApp):
         self.progress._window.set_focus_on_map(False)
     # show the main window
     self.window_main.show()
+    # get the install backend
+    self.install_backend = backend.backend_factory(self.window_main)
+
     # it can only the iconified *after* it is shown (even if the docs
     # claim otherwise)
     if options.no_focus_on_map:
         self.window_main.iconify()
+        self.window_main.stick()
         self.window_main.set_urgency_hint(True)
-        self.window_main.connect("focus-in-event",
-                                 lambda w,e: (w.set_urgency_hint(False) and False))
+        self.initial_focus_id = self.window_main.connect(
+            "focus-in-event", self.on_initial_focus_in)
+    else:
+        self.warn_on_battery()
 
+  def on_initial_focus_in(self, widget, event):
+      """callback run on initial focus-in (if started unmapped)"""
+      widget.unstick()
+      widget.set_urgency_hint(False)
+      self.window_main.disconnect(self.initial_focus_id)
+      self.warn_on_battery()
+      return False
+
+  def warn_on_battery(self):
+      """check and warn if on battery"""
+      if on_battery():
+          self.dialog_on_battery.set_transient_for(self.window_main)
+          res = self.dialog_on_battery.run()
+          self.dialog_on_battery.hide()
+          if res != gtk.RESPONSE_YES:
+              sys.exit()
 
   def install_column_view_func(self, cell_layout, renderer, model, iter):
     pkg = model.get_value(iter, LIST_PKG)
@@ -280,8 +309,7 @@ class UpdateManager(SimpleGladeApp):
         
 
   def on_treeview_update_cursor_changed(self, widget):
-    tuple = widget.get_cursor()
-    path = tuple[0]
+    path = widget.get_cursor()[0]
     # check if we have a path at all
     if path == None:
       return
@@ -299,9 +327,6 @@ class UpdateManager(SimpleGladeApp):
       return
     long_desc = pkg.description
     self.notebook_details.set_sensitive(True)
-    # Skip the first line - it's a duplicate of the summary
-    i = long_desc.find("\n")
-    long_desc = long_desc[i+1:]
     # do some regular expression magic on the description
     # Add a newline before each bullet
     p = re.compile(r'^(\s|\t)*(\*|0|-)',re.MULTILINE)
@@ -348,6 +373,14 @@ class UpdateManager(SimpleGladeApp):
         # download finished (or canceld, or time-out)
         button.hide()
         button.disconnect(id);
+    # check if we still are in the right pkg (the download may have taken
+    # some time and the user may have clicked on a new pkg)
+    path  = widget.get_cursor()[0]
+    if path == None:
+      return
+    now_name = widget.get_model()[path][LIST_NAME]
+    if name != now_name:
+        return
     # display NEWS.Debian first, then the changelog
     changes = ""
     srcpkg = self.cache[name].sourcePackageName
@@ -412,10 +445,11 @@ class UpdateManager(SimpleGladeApp):
   def refresh_updates_count(self):
       self.button_install.set_sensitive(self.cache.installCount)
       try:
+          inst_count = self.cache.installCount
           self.dl_size = self.cache.requiredDownload
-          # TRANSLATORS: b stands for Bytes
-          self.label_downsize.set_markup(_("Download size: %s") % \
-                                         humanize_size(self.dl_size))
+          t = _("%s selected. Download size: %s") % (inst_count,
+                                                       humanize_size(self.dl_size))
+          self.label_downsize.set_text(t)
       except SystemError, e:
           print "requiredDownload could not be calculated: %s" % e
           self.label_downsize.set_markup(_("Unknown download size"))
@@ -446,6 +480,18 @@ class UpdateManager(SimpleGladeApp):
           return _("The package information was last updated less than one hour ago.")
       return None
 
+  def update_last_updated_text(self):
+      """timer that updates the last updated text """
+      #print "update_last_updated_text"
+      num_updates = self.cache.installCount
+      if num_updates == 0:
+          if self._get_last_apt_get_update_text() is not None:
+              text_label_main = self._get_last_apt_get_update_text()
+              self.label_main_details.set_text(text_label_main)
+          return True
+      # stop the timer if there are upgrades now
+      return False
+
   def update_count(self):
       """activate or disable widgets and show dialog texts correspoding to
          the number of available updates"""
@@ -454,16 +500,18 @@ class UpdateManager(SimpleGladeApp):
       text_label_main = _("Software updates correct errors, eliminate security vulnerabilities and provide new features.")
       if num_updates == 0:
           text_header= "<big><b>%s</b></big>"  % _("Your system is up-to-date")
-          text_download = ""
+          self.label_downsize.set_text("")
           self.notebook_details.set_sensitive(False)
           self.treeview_update.set_sensitive(False)
           self.button_install.set_sensitive(False)
-          self.label_downsize.set_text=""
           self.button_close.grab_default()
           self.textview_changes.get_buffer().set_text("")
           self.textview_descr.get_buffer().set_text("")
           if self._get_last_apt_get_update_text() is not None:
               text_label_main = self._get_last_apt_get_update_text()
+          # add timer to ensure we update the information when the 
+          # last package count update was performed
+          glib.timeout_add_seconds(10, self.update_last_updated_text)
       else:
           # show different text on first run (UX team suggestion)
           firstrun = self.gconfclient.get_bool("/apps/update-manager/first_run")
@@ -474,14 +522,13 @@ class UpdateManager(SimpleGladeApp):
           else:
               text_header = "<big><b>%s</b></big>" % _("Software updates are available for this computer")
               text_label_main = _("If you don't want to install them now, choose \"Update Manager\" from the Administration menu later.")
-          text_download = _("Download size: %s") % humanize_size(self.dl_size)
           self.notebook_details.set_sensitive(True)
           self.treeview_update.set_sensitive(True)
           self.button_install.grab_default()
           self.treeview_update.set_cursor(1)
       self.label_header.set_markup(text_header)
-      self.label_downsize.set_markup(text_download)
       self.label_main_details.set_text(text_label_main)
+      return True
 
   def activate_details(self, expander, data):
     expanded = self.expander_details.get_expanded()
@@ -494,44 +541,6 @@ class UpdateManager(SimpleGladeApp):
     if expanded:
       self.on_treeview_update_cursor_changed(self.treeview_update)
 
-  def run_synaptic(self, id, action, lock):
-    try:
-      apt_pkg.PkgSystemUnLock()
-    except SystemError:
-      pass
-    cmd = ["/usr/bin/gksu", 
-           "--desktop", "/usr/share/applications/update-manager.desktop", 
-           "--", "/usr/sbin/synaptic", "--hide-main-window",  
-           "--non-interactive", "--parent-window-id", "%s" % (id) ]
-    if action == INSTALL:
-      # close when update was successful (its ok to use a Synaptic::
-      # option here, it will not get auto-saved, because synaptic does
-      # not save options in non-interactive mode)
-      if self.gconfclient.get_bool("/apps/update-manager/autoclose_install_window"):
-          cmd.append("-o")
-          cmd.append("Synaptic::closeZvt=true")
-      # custom progress strings
-      cmd.append("--progress-str")
-      cmd.append("%s" % _("Please wait, this can take some time."))
-      cmd.append("--finish-str")
-      cmd.append("%s" %  _("Update is complete"))
-      f = tempfile.NamedTemporaryFile()
-      for pkg in self.cache:
-          if pkg.markedInstall or pkg.markedUpgrade:
-              f.write("%s\tinstall\n" % pkg.name)
-      cmd.append("--set-selections-file")
-      cmd.append("%s" % f.name)
-      f.flush()
-      subprocess.call(cmd)
-      f.close()
-    elif action == UPDATE:
-      cmd.append("--update-at-startup")
-      subprocess.call(cmd)
-    else:
-      print "run_synaptic() called with unknown action"
-      sys.exit(1)
-    lock.release()
-
   def on_button_reload_clicked(self, widget):
     #print "on_button_reload_clicked"
     self.check_metarelease()
@@ -541,22 +550,22 @@ class UpdateManager(SimpleGladeApp):
   #  self.help_viewer.run()
 
   def on_button_settings_clicked(self, widget):
-      #print "on_button_settings_clicked"
-      try:
-          apt_pkg.PkgSystemUnLock()
-      except SystemError:
-          pass
-      cmd = ["/usr/bin/gksu", 
-             "--desktop", "/usr/share/applications/software-properties.desktop", 
-             "--", "/usr/bin/software-properties-gtk","--open-tab","2",
-             "--toplevel", "%s" % self.window_main.window.xid ]
-      self.window_main.set_sensitive(False)
-      p = subprocess.Popen(cmd)
-      while p.poll() is None:
-          while gtk.events_pending():
-              gtk.main_iteration()
-          time.sleep(0.05)
-      self.fillstore()
+    #print "on_button_settings_clicked"
+    try:
+        apt_pkg.PkgSystemUnLock()
+    except SystemError:
+        pass
+    cmd = ["/usr/bin/gksu", 
+           "--desktop", "/usr/share/applications/software-properties.desktop", 
+           "--", "/usr/bin/software-properties-gtk","--open-tab","2",
+           "--toplevel", "%s" % self.window_main.window.xid ]
+    self.window_main.set_sensitive(False)
+    p = subprocess.Popen(cmd)
+    while p.poll() is None:
+        while gtk.events_pending():
+            gtk.main_iteration()
+        time.sleep(0.05)
+    self.fillstore()
 
   def on_button_install_clicked(self, widget):
     #print "on_button_install_clicked"
@@ -591,16 +600,13 @@ class UpdateManager(SimpleGladeApp):
     # set window to insensitive
     self.window_main.set_sensitive(False)
     self.window_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-    lock = thread.allocate_lock()
-    lock.acquire()
-    t = thread.start_new_thread(self.run_synaptic,
-                                (self.window_main.window.xid,action,lock))
-    while lock.locked():
-      while gtk.events_pending():
-        gtk.main_iteration()
-      time.sleep(0.05)
-    while gtk.events_pending():
-      gtk.main_iteration()
+
+    # do it
+    if action == UPDATE:
+        self.install_backend.update()
+    elif action == INSTALL:
+        self.install_backend.commit(self.cache)
+
     s = _("Reading package information")
     self.label_cache_progress_title.set_label("<b><big>%s</big></b>" % s)
     self.fillstore()
@@ -658,11 +664,26 @@ class UpdateManager(SimpleGladeApp):
     self.setBusy(False)
 
   def on_treeview_update_row_activated(self, treeview, path, column, *args):
-      """
-      If an update row was activated (by pressing space), toggle the 
-      install check box
-      """
-      self.toggled(None, path)
+    """
+    If an update row was activated (by pressing space), toggle the 
+    install check box
+    """
+    self.toggled(None, path)
+
+  def on_window_main_size_allocate(self,arg1,arg2):
+    """ 
+    recalculates headline labels on window resize to work around
+    problems with gtk word wrapping 
+    (http://bugzilla.gnome.org/show_bug.cgi?id=101968)
+    """
+    # this number is based on border width 
+    # (2*main_border_with + icon width + 2*icon_border_with)
+    #     2*18 + 48 + 2*12 
+    border_space = 96
+    width, height = self.window_main.get_size()
+    #print "on_window_main_size_allocate", width, height
+    self.label_main_details.set_size_request(width - border_space ,-1)
+    self.label_header.set_size_request(width - border_space,-1)
 
   def exit(self):
     """ exit the application, save the state """
@@ -748,6 +769,8 @@ class UpdateManager(SimpleGladeApp):
                            origin.description, None, origin])
         for pkg in self.list.pkgs[origin]:
           name = xml.sax.saxutils.escape(pkg.name)
+          if not pkg.isInstalled:
+              name += _(" (New install)")
           summary = xml.sax.saxutils.escape(pkg.summary)
           contents = "<b>%s</b>\n<small>%s</small>" % (name, summary)
           #TRANSLATORS: the b stands for Bytes
@@ -770,16 +793,18 @@ class UpdateManager(SimpleGladeApp):
 
   def dist_no_longer_supported(self, meta_release):
     msg = "<big><b>%s</b></big>\n\n%s" % \
-          (_("Your distribution is not supported anymore"),
+          (_("Your Ubuntu release is not supported anymore"),
 	   _("You will not get any further security fixes or critical "
              "updates. "
-             "Upgrade to a later version of Ubuntu Linux. See "
-             "http://www.ubuntu.com for more information on "
-             "upgrading."))
+             "Please Upgrade to a later version of Ubuntu Linux."))
     dialog = gtk.MessageDialog(self.window_main, 0, gtk.MESSAGE_WARNING,
                                gtk.BUTTONS_CLOSE,"")
     dialog.set_title("")
     dialog.set_markup(msg)
+    button = gtk.LinkButton(_("Upgrade information"))
+    button.set_uri("http://www.ubuntu.com/getubuntu/upgrading")
+    button.show()
+    dialog.get_content_area().pack_end(button)
     dialog.run()
     dialog.destroy()
 
@@ -803,7 +828,7 @@ class UpdateManager(SimpleGladeApp):
       
   def new_dist_available(self, meta_release, upgradable_to):
     self.frame_new_release.show()
-    self.label_new_release.set_markup(_("<b>New distribution release '%s' is available</b>") % upgradable_to.version)
+    self.label_new_release.set_markup(_("<b>New Ubuntu release '%s' is available</b>") % upgradable_to.version)
     self.new_dist = upgradable_to
     
 
